VERSION 1.0 CLASS
BEGIN
  MultiUse = -1  'True
  Persistable = 0  'NotPersistable
  DataBindingBehavior = 0  'vbNone
  DataSourceBehavior  = 0  'vbNone
  MTSTransactionMode  = 0  'NotAnMTSObject
END
Attribute VB_Name = "ExcelWorkbook"
Attribute VB_GlobalNameSpace = False
Attribute VB_Creatable = True
Attribute VB_PredeclaredId = False
Attribute VB_Exposed = False

'CLASS MODULE TO OPEN AND MANIPULATE AN EXCEL WORKBOOK.

'There should be nothing here that is specific to any
'particular workbook.

' Philip Bolt, 2005
'Last updated: 16/4/05

'The main program must have:-
'   a Public boolean, 'canceled' which is used to cancel
'   Method and Property calls made before there is an open workbook
'   or in case of an error and
'   A procedure, 'select_sheet_action' to handle the return from
'   SheetActivate' events.

Option Explicit
Option Compare Text

Public name As String                   'Book name, becomes object property.
Dim WithEvents xlBk As Excel.Workbook   'Define the workbook object.
Attribute xlBk.VB_VarHelpID = -1
Private opened As Boolean               'Define the 'open Excel workbook' flag.

'Object constructor.
'Run when the object is created.

Private Sub Class_Initialize()
 
    opened = False       'Flag that a workbook has NOT been opened.
  
End Sub

'Class method to open a WORKBOOK specified by name.
'Method name:   open_book(nme)
'Parameter:     nme;    the workbook name.

Public Sub open_book(nme As String)

    If Trim(nme) = "" Then
        MsgBox "There is a blank for the book name!", vbOKOnly + vbCritical, "'open_book' ERROR"
        Exit Sub
    End If
    
    On Error GoTo etrap     'Set the error trap.
    
    Set xlBk = GetObject(nme)               'Define the book object variable and
    xlBk.Windows(xlBk.name).Activate        'activate the workbook.
    xlBk.Application.WindowState = xlNormal 'Set the windows to normal size,
    xlBk.Application.Visible = True         'make it visible
    xlBk.Parent.Windows(1).Visible = True   'with its main window displayed.
    
    opened = True       'Flag that there is an open workbook,
    name = xlBk.name    'store the book name.
    On Error GoTo 0     'Clear the 'On error' jump.
    
Exit Sub
    
etrap: 'ERROR TRAP
    error_report Err.Number, nme, "'open_book'"
    
End Sub

'Class property to return an object which is the open workbook.
'Property name: get_book()

Public Property Get get_book()

    If opened Then
        On Error GoTo etrap         'Set error trap.
        
        Set get_book = xlBk         'Return the book object.
        
        On Error GoTo 0             'Clear the 'On error' jump.
    Else
        canceled = True             'Set the CANCELed flag and
        Set get_book = Nothing      'return nothing.
    End If
 
    Exit Property
    
etrap: 'ERROR TRAP
    error_report Err.Number, "", "'get_book'"
    
    Set get_book = Nothing             'Return nothing.
 
End Property

'Class property to return the name of the active sheet.
'Property name: sheet_name

Public Property Get sheet_name()

    If opened Then
        sheet_name = xlBk.ActiveSheet.name
    Else
        canceled = True
        sheet_name = ""
    End If
 
End Property

'Class property to return an object which is the current sheet.
'Property name: current_name

Public Property Get get_sheet()

    If opened Then                          'If a book has been opened
        Set get_sheet = xlBk.ActiveSheet    'return the active sheet object.
    Else                                    'Otherwise,
        canceled = True                     'set the CANCELed flag and
        Set get_sheet = Nothing             'return nothing.
    End If
    
End Property

'Class property to select a worksheet specified by name WITHOUT selecting it.
'Property name: set_sheet(s_nme)
'Parameter:     s_nme;  the worksheet name.

Public Property Get set_sheet(s_nme As String)

    Dim cs As Object
    
    If opened Then
    
        On Error GoTo etrap
    
        Set set_sheet = xlBk.Sheets(s_nme)   'Define the sheet variable.
 
        On Error GoTo 0             'Clear the 'On error' jump.
    Else
        canceled = True             'Set the CANCELed flag.
        Set set_sheet = Nothing
    End If
    
    Exit Property
    
etrap: 'ERROR TRAP
        error_report Err.Number, s_nme, "'select_sheet'"
        Set set_sheet = Nothing
        
End Property

'Class method to clear the current WORKSHEET.
'Method name:   clear_sheet()

Public Sub clear_sheet()

    Dim rn, cs As Object, r, c As Integer
    
    If opened Then
        Set cs = xlBk.ActiveSheet   'Define the sheet object variable and
        cs.Select                   'select it.
    
        r = Val(cs.Cells(1, 12)): c = Val(cs.Cells(1, 13)) 'The clearance values are in A12, A13.

        If r < 1 Or c < 1 Then
            MsgBox "One of the clearance values is zero or negative.", vbOKOnly + vbCritical, "'clear_sheet' ERROR"
            canceled = True     'Set the CANCELed flag.
            Exit Sub
        End If
    
        On Error GoTo etrap             'Set error trap.
    
        Set rn = cs.Range(cs.Cells(1, 1), cs.Cells(r, c))   'Define the range and
        rn.ClearContents                                    'clear the contents.
    
        On Error GoTo 0     'Clear the 'On error' jump.
    Else
        canceled = True     'Set the CANCELed flag.
    End If
    
    Exit Sub
    
etrap: 'ERROR TRAP
    error_report Err.Number, "", "'clear_sheet'"
    
End Sub

'Class property to return an object which is a specified range of cells
'on the specified sheet.
'Property name: set_range(s_nme, r1, c1, r2, c2)
'Parameter:     s_nme;      the sheet name.
'               (r1, c1);   the TL cell,
'               (r2, c2);   the BR cell.


Public Property Get set_range(s_nme As String, r1 As Integer, c1 As Integer, r2 As Integer, c2 As Integer)

    Dim rn, cs As Object
    
    If opened Then
        If r1 < 1 Or c1 < 1 Or r2 < 1 Or c2 < 1 Then
            MsgBox "One of the cell values is zero or negative.", vbOKOnly + vbCritical, "'set_range' ERROR"
            canceled = True             'Set the CANCELed flag.
            Set set_range = Nothing     'Return nothing.
            Exit Property
        End If
 
        On Error GoTo etrap             'Set error trap.
    
        Set cs = xlBk.Sheets(s_nme)     'Define the sheet object variable and
        Set rn = cs.Range(cs.Cells(r1, c1), cs.Cells(r2, c2))   'the range.

        Set set_range = rn              'Return the range.

        On Error GoTo 0                 'Clear the 'On error' jump.
    Else
        canceled = True
        Set set_range = Nothing         'Return nothing.
    End If

Exit Property
    
etrap: 'ERROR TRAP
    error_report Err.Number, s_nme, "'set_range'"
    
End Property

'Class property to return an object which is the specified active sheet.
'Property name: select_sheet(s_nme)
'Parameter:     s_nme;  the sheet name.

Public Property Get select_sheet(s_nme As String)

    Dim cs As Object
    
    If opened Then
        On Error GoTo etrap                 'Set error trap.
    
        Set cs = xlBk.Sheets(s_nme)         'Set the sheet object variable to the named sheet,
        cs.Select                           'select it and
        Set select_sheet = xlBk.ActiveSheet    'return the object.

        On Error GoTo 0                 'Clear the 'On error' jump.
    Else
        canceled = True                 'Set the CANCELed flag and
        Set select_sheet = Nothing         'return nothing.
    End If
    
    Exit Property
    
etrap: 'ERROR TRAP
    error_report Err.Number, s_nme, "'set_sheet'"
    
    Set select_sheet = Nothing             'Return nothing.
 
End Property

'Class method to save the current workbook.
'Method name:   save_book

Public Sub save_book()

    If opened Then
        If Not xlBk.Saved Then
            xlBk.Save   'If changes made, save the book.
        End If
    Else
        canceled = True 'Set the CANCELed flag.
    End If

End Sub

'Class method to close the workbook.
'Method name:   close_book

Public Sub close_book()

    If opened Then
        If Not xlBk.Saved Then
            xlBk.Save       'Save the book if changes made,
        End If
        xlBk.Close          'close it and
        Set xlBk = Nothing  'release the handle.
        opened = False      'Flag that this book is closed.
    Else
        canceled = True     'Set the CANCELed flag.
    End If

End Sub

'Object terminator.
'This runs whenever the object is terminated, (Class variable = Nothing).
'including when the package is closed for ONLY ONE object.

Private Sub Class_Terminate()

    If opened Then
        xlBk.Save
        xlBk.Close
        Set xlBk = Nothing  'Release the handle.
    End If
    
End Sub

'Routine run whenever a sheet is selected.
'This passes the sheet name to an external procedure
'which allows the actual actions to be tailored to the project concerned.
'Procedure name:    xlBk_SheetActivate
'Parameter:         Sh; the Sheet object.

Private Sub xlBk_SheetActivate(ByVal Sh As Object)

   select_sheet_action Sh.name, xlBk.name   'Run the package routine,
                                            'passing the sheet & book name.
End Sub

'Error routine to report the various errors.
'Procedure name:    error_report(e_num, s_nm, mp_nme)
'Parameter:         e_num;  the error number,
'                   s_nm;   the sheet/book name, (where applicable).
'                   mp_nme; the Method/Property name.

Private Sub error_report(e_num, s_nm, mp_nme)

    Dim txt As String
    
    Select Case e_num
    Case 9
        txt = "Sheet '" + s_nm + "' doesn't exist."
    Case 13
        txt = "A clearance cell contains non-numeric data."
    Case 432
        txt = "Workbook '" + s_nm + "' doesn't exist."
    Case Else
        txt = "Error # " & Str(Err.Number) & " was generated by " _
         & vbCrLf & Err.Source & vbCrLf & Err.Description
    End Select
    
    MsgBox txt, vbOKOnly + vbCritical, mp_nme + " ERROR"
        
    Err.Clear       'Clear the error object.

    canceled = True 'Set the CANCELed flag.
    
End Sub
